"""
App init
"""

# Django
from django.utils.translation import gettext_lazy as _

__version__ = "2.5.0"
__title__ = _("Sovereignty Timers")

__package_name__ = "aa-sov-timer"
__app_name__ = "sovtimer"
__app_name_verbose__ = "AA Sovereignty Timer"
__app_name_useragent__ = "AA-Sovereignty-Timer"

__github_url__ = f"https://github.com/ppfeufer/{__package_name__}"
